unit uMainVisualFrame;
{< for visual coding of component}

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Error, FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Stan.Async,
  FireDAC.Phys, FireDAC.VCLUI.Wait, Data.DB, FireDAC.Comp.Client, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.Buttons,
  FireDAC.Stan.Param,


  System.SyncObjs,
  System.Contnrs,

  //myUnits

//  uDBConnection_Coffeetest,
  uPictureFrame,
  uQueries,
  uDBPagination,
  uGalleryOfPictureFrames,
  uCheckFileType,
  uHTTP_ConnectionParams_DBGalleryViewer,
  uSearch_DBGalleryViewer,
  uHTTPRequests_DBGalleryViewer,

   Vcl.Grids, Vcl.DBGrids, System.ImageList, Vcl.ImgList, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient,
  IdHTTP, Vcl.AppEvnts, Vcl.ComCtrls, Vcl.Samples.Spin

  ;

type
  TMainVisualFrame = class(TFrame)
    FDConnection: TFDConnection;
    pTopPanel: TPanel;
    bTest: TButton;
    ScrollBox: TScrollBox;
    ilDefaultPics: TImageList;
    IdHTTPDownloadPicture: TIdHTTP;
    ApplicationEvents: TApplicationEvents;
    pNavPanel: TPanel;
    bRight: TBitBtn;
    pElementsOnPage: TPanel;
    lPages: TLabel;
    bLeft: TBitBtn;
    cbElementsOnPage: TComboBox;
    lElementsOnPage: TLabel;
    ProgressBar: TProgressBar;
    ilArrows16x16: TImageList;
    sePages: TSpinEdit;
    lPageNumber: TLabel;
    bOpenPage: TBitBtn;
    pSort: TPanel;
    cbSortParam: TComboBox;
    cbSortDirection: TComboBox;
    bSort: TBitBtn;
    pSearch: TPanel;
    pSearchTop: TPanel;
    bSearch: TBitBtn;
    eSearch: TEdit;
    pHomeUpdateButtons: TPanel;
    bHome: TBitBtn;
    bUpdate: TBitBtn;
    bFlushSorting: TBitBtn;
    cbSearchType: TComboBox;
    bFlushSearch: TBitBtn;

    procedure FrameResize(Sender: TObject);
    procedure bTestClick(Sender: TObject);
    procedure bRightClick(Sender: TObject);
    procedure bLeftClick(Sender: TObject);
    procedure cbElementsOnPageChange(Sender: TObject);
    procedure ApplicationEventsException(Sender: TObject; E: Exception);
    procedure FrameMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure FrameMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure bOpenPageClick(Sender: TObject);
    procedure sePagesChange(Sender: TObject);
    procedure bSortClick(Sender: TObject);
    procedure bHomeClick(Sender: TObject);
    procedure bUpdateClick(Sender: TObject);
    procedure bSearchClick(Sender: TObject);
    procedure bFlushSearchClick(Sender: TObject);
    procedure cbSortDirectionChange(Sender: TObject);
    procedure eSearchClick(Sender: TObject);
    procedure bFlushSortingClick(Sender: TObject);

  private
    { Private declarations }


    FQueries:TQueries;
    FIDUser:Integer;
    FDBPagination:TDBPagination;
    FGalleryOfPictureFrames:TGalleryOfPictureFrames;
    FCheckFileType:TCheckFileType;
    FHTTP_ConnectionParams:THTTP_ConnectionParams;
    FCSDownloadPictureThread:TCriticalSection;

    FDownloadThreadsOL:TObjectList;

    FSorted:Boolean;
    FSearch:Boolean;

    FSearch_DBGalleryViewer:TSearch_DBGalleryViewer;

    FHTTPRequests_DBGalleryViewer:THTTPRequests_DBGalleryViewer;

    FFirstSearchPhraseDeleted:Boolean;
    
    procedure CenterElements;
    procedure UpdatePagesLabel;
    procedure DisposeGalleryOfPictureFrames;


    function  IsAllDownloadThreadsFinished:boolean;
    procedure FinishAllDownLoadThreads;
    procedure CheckIfAllDownloadThreadsFinished;
    procedure LoadPage(APageNumber: integer);







  public
    { Public declarations }

    constructor Create(AOwner:TComponent); override;
    destructor destroy; override;


    procedure ConnectAllQueries();

    procedure SetupDBPaginationModule();


    procedure LoadFirstPage();
    procedure LoadNextPage();
    procedure LoadPreviousPage();




    property IDUser:Integer read FIDUser write FIDUser;
    procedure UpdateObjectListOfPictures;

    procedure UpdateNumberOfRecords;

    procedure SortIfNeeded();


    property Queries:TQueries read FQueries write FQueries;

    property DBPagination:TDBPagination read FDBPagination write FDBPagination;

    property HTTP_ConnectionParams:THTTP_ConnectionParams read FHTTP_ConnectionParams
    write FHTTP_ConnectionParams;

    property CheckFileType:TCheckFileType read FCheckFileType write FCheckFileType;

    property GalleryOfPictureFrames:TGalleryOfPictureFrames read FGalleryOfPictureFrames
    write FGalleryOfPictureFrames;

    property CSDownloadPictureThread:TCriticalSection read FCSDownloadPictureThread
    write FCSDownloadPictureThread;

    property Sorted:Boolean read FSorted write FSorted;

  end;

implementation

{$R *.dfm}

uses uDownLoadPictureThread_DBGalleryViewer;

{ TMainVisualFrame }

procedure TMainVisualFrame.ApplicationEventsException(Sender: TObject; E: Exception);
begin

//

// TDownloadFileThread Exception catching
if (Sender is TDownloadPictureThread ) then

begin

      ShowMessage(

      'Error Raised '+E.ClassName+'Error Message : '+
                  E.Message

                  );

end;







end;



procedure TMainVisualFrame.CheckIfAllDownloadThreadsFinished;
begin


if not IsAllDownloadThreadsFinished then begin

  case Application.MessageBox('   ,  ?',
    ' ', MB_YESNO + MB_ICONQUESTION) of
    IDYES:
      begin

          FinishAllDownLoadThreads;


      end;
    IDNO:
      begin
      exit;
      end;
  end;


end;



end;





procedure TMainVisualFrame.bFlushSearchClick(Sender: TObject);
begin
bHomeClick(Self);
eSearch.Text:='';
end;

procedure TMainVisualFrame.bFlushSortingClick(Sender: TObject);
begin

FSorted:=False;
LoadFirstPage();

end;

procedure TMainVisualFrame.bHomeClick(Sender: TObject);
begin


FSearch:=false;
LoadFirstPage();
eSearch.Text:='';
end;

procedure TMainVisualFrame.bLeftClick(Sender: TObject);
begin
FHTTPRequests_DBGalleryViewer.IsHTTPConnectionOk();

//CheckIfAllDownloadThreadsFinished;

LoadPreviousPage;
end;

procedure TMainVisualFrame.bOpenPageClick(Sender: TObject);

begin

FHTTPRequests_DBGalleryViewer.IsHTTPConnectionOk();

if sePages.Value>DBPagination.CountOfPages then
sePages.Value:=DBPagination.CountOfPages;

if sePages.Value<1 then sePages.Value:=1;


if not IsAllDownloadThreadsFinished then begin
ShowMessage('  ,     ');
exit;
end;



if Assigned(FDBPagination) then FreeAndNil(FDBPagination);
FDBPagination:=TDBPagination.Create(Self);

SetupDBPaginationModule();


DBPagination.LoadPage(sePages.Value); // << opens query, load records

UpdatePagesLabel();
UpdateObjectListOfPictures();
FGalleryOfPictureFrames.DisposePicturesOnGalleryFrame();




end;

procedure TMainVisualFrame.bRightClick(Sender: TObject);
begin

FHTTPRequests_DBGalleryViewer.IsHTTPConnectionOk();
//CheckIfAllDownloadThreadsFinished;

LoadNextPage();
end;

procedure TMainVisualFrame.bSearchClick(Sender: TObject);

var TextToSearch:string;

begin

TextToSearch:=Trim(eSearch.Text);
if TextToSearch='' then Exit;
 
 FSearch:=True;
 LoadFirstPage();

end;

procedure TMainVisualFrame.bSortClick(Sender: TObject);
begin

 FSorted:=true;
 LoadFirstPage();

end;

procedure TMainVisualFrame.bTestClick(Sender: TObject);
begin

{
if not IsAllDownloadThreadsFinished then begin
ShowMessage('  ,     ');
exit;
end;
}


LoadFirstPage();

//DBGrid.DataSource.DataSet:=FDBPagination.FDQuery;
{
DataSource.DataSet:=FDBPagination.FDQuery;

FDBPagination.LoadFirstPage();
  UpdatePagesLabel;

}

{
  with FQueries.qSelectAllFilesOfUser do begin

   Disconnect();

   Params.ParamValues['users_id']:=FIDUser;

   Open();

   ShowMessage( FieldByName('name').AsString );


  end;

}

end;

procedure TMainVisualFrame.bUpdateClick(Sender: TObject);
begin
LoadPage(DBPagination.CurrentPage);
end;

procedure TMainVisualFrame.cbElementsOnPageChange(Sender: TObject);
begin
LoadFirstPage();
end;

procedure TMainVisualFrame.cbSortDirectionChange(Sender: TObject);
begin

 case cbSortDirection.ItemIndex of

  0: 
  begin

  bSort.Glyph:=nil;
  ilArrows16x16.GetBitmap(7,bSort.Glyph);

  end;
  1: begin

  bSort.Glyph:=nil;
  ilArrows16x16.GetBitmap(8,bSort.Glyph);

  end;
   
   
 end;

end;

procedure TMainVisualFrame.CenterElements;
begin

lPages.Left:=Trunc(pNavPanel.Width/2 - lPages.Width/2);

bLeft.Left:=lPages.Left-bLeft.Width-15;
bRight.Left:=lPages.Left+lPages.Width+15;


bLeft.Top:=trunc(pNavPanel.Height/2-bLeft.Height/2);
bRight.Top:=trunc(pNavPanel.Height/2-bRight.Height/2);

bHome.Top:=trunc(pNavPanel.Height/2-bHome.Height/2);
bUpdate.Top:=trunc(pNavPanel.Height/2-bUpdate.Height/2);

bHome.Left:=7;
bUpdate.Left:=bHome.Left+bHome.Width+7;

end;

procedure TMainVisualFrame.ConnectAllQueries;
var
  i: Integer;
begin

//

// Connecting queries
for i := 0 to FQueries.ComponentCount-1 do begin

 if (FQueries.Components[i] is TFDQuery) then
 (FQueries.Components[i] as TFDQuery).Connection:=FDConnection;

end;


// Connecting FCheckFileType queries
for i := 0 to FCheckFileType.ComponentCount-1 do begin

 if (FCheckFileType.Components[i] is TFDQuery) then
 (FCheckFileType.Components[i] as TFDQuery).Connection:=FDConnection;

end;


end;

constructor TMainVisualFrame.Create(AOwner: TComponent);


begin
  inherited;
//

CenterElements;
FQueries:=TQueries.Create(Self);

FDBPagination:=TDBPagination.Create(Self);
FGalleryOfPictureFrames:=TGalleryOfPictureFrames.Create(Self);
FGalleryOfPictureFrames.Parent:=ScrollBox;
FCheckFileType:=TCheckFileType.Create(Self);
FHTTP_ConnectionParams:=THTTP_ConnectionParams.Create(Self);

DisposeGalleryOfPictureFrames;


FCSDownloadPictureThread:=TCriticalSection.Create;

FDownloadThreadsOL:=TObjectList.Create;
FDownloadThreadsOL.OwnsObjects:=true;


FSearch_DBGalleryViewer:=TSearch_DBGalleryViewer.Create(Self);


FHTTPRequests_DBGalleryViewer:=THTTPRequests_DBGalleryViewer.Create(Self);

bLeft.Caption:='';
bRight.Caption:='';

ilArrows16x16.GetBitmap(0,bLeft.Glyph);
ilArrows16x16.GetBitmap(1,bRight.Glyph);

bOpenPage.Caption:='';
ilArrows16x16.GetBitmap(2,bOpenPage.Glyph);


bHome.Caption:='';
ilArrows16x16.GetBitmap(3,bHome.Glyph);

bUpdate.Caption:='';
ilArrows16x16.GetBitmap(4,bUpdate.Glyph);

bSearch.Caption:='';
ilArrows16x16.GetBitmap(5,bSearch.Glyph);

bFlushSearch.Caption:='';
ilArrows16x16.GetBitmap(6,bFlushSearch.Glyph);

bFlushSorting.Caption:='';
ilArrows16x16.GetBitmap(6,bFlushSorting.Glyph);

//bSort.Caption:='';
ilArrows16x16.GetBitmap(8,bSort.Glyph);

FSorted:=true;
FSearch:=false;

eSearch.Text:='    ...';


end;


destructor TMainVisualFrame.destroy;
begin

//FinishAllDownLoadThreads();
FreeAndNil(FDownloadThreadsOL);
FreeAndNil(FCSDownloadPictureThread);

  inherited;
end;

procedure TMainVisualFrame.FinishAllDownLoadThreads;
var i:integer;

begin

for i := FDownloadThreadsOL.Count-1 to 0 do begin

//TDownloadPictureThread(FDownloadThreadsOL[i]).Terminate;
//TDownloadPictureThread(FDownloadThreadsOL[i]).WaitFor;
//TDownloadPictureThread(FDownloadThreadsOL[i]).Free;

FDownloadThreadsOL.Delete(i); // deleting link from object list

end;



end;

procedure TMainVisualFrame.FrameMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint;
  var Handled: Boolean);
begin

ScrollBox.VertScrollBar.Position:=
ScrollBox.VertScrollBar.Position+4;

end;

procedure TMainVisualFrame.FrameMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint;
  var Handled: Boolean);
begin

ScrollBox.VertScrollBar.Position:=
ScrollBox.VertScrollBar.Position-4;

end;

procedure TMainVisualFrame.FrameResize(Sender: TObject);
begin

CenterElements;
DisposeGalleryOfPictureFrames;


ScrollBox.Height:=
Self.Height-pTopPanel.Height-ProgressBar.Height;

if pSort.Visible then
ScrollBox.Height:=ScrollBox.Height-pSort.Height;

if pSearch.Visible then
ScrollBox.Height:=ScrollBox.Height-pSearch.Height;

FGalleryOfPictureFrames.DisposePicturesOnGalleryFrame;

if ScrollBox.Visible then
ScrollBox.SetFocus;


//

end;



function TMainVisualFrame.IsAllDownloadThreadsFinished: boolean;
var
  i: Integer;
begin
//
Result:=True;

for i := 0 to FDownloadThreadsOL.Count-1 do begin

 if not TDownloadPictureThread(FDownloadThreadsOL[i]).Finished then
  begin
    Result:=False;
    Break;
  end;


end;



end;

procedure TMainVisualFrame.LoadFirstPage;

var TextToSearch:string;

begin

FHTTPRequests_DBGalleryViewer.IsHTTPConnectionOk();

  if not IsAllDownloadThreadsFinished then begin
  ShowMessage('  ,     ');
  exit;
  end;



  if Assigned(FDBPagination) then FreeAndNil(FDBPagination);
  FDBPagination:=TDBPagination.Create(Self);

  if FSearch then begin
  TextToSearch:=Trim(eSearch.Text );
  //TextToSearch:=AnsiLowerCase(TextToSearch);

  case cbSearchType.ItemIndex of

     0: begin
     FSearch_DBGalleryViewer.SetupDBPaginationModuleEntrySearch( TextToSearch );

         if FDBPagination.CountOfRecords=0 then begin
         ShowMessage('  ');
         Exit();
         end;


     end;

     1: begin

     FSearch_DBGalleryViewer.SetupDBPaginationModuleExactSearch( TextToSearch );

         if FDBPagination.CountOfRecords=0 then begin
         ShowMessage('  ');
         Exit();
         end;


     end;


  end;


  end

   else
  SetupDBPaginationModule();

  DBPagination.LoadFirstPage(); // << opens query, load records

  UpdatePagesLabel();
  UpdateObjectListOfPictures();
  FGalleryOfPictureFrames.DisposePicturesOnGalleryFrame();

  sePages.Value:=1;


end;




procedure TMainVisualFrame.LoadPage(APageNumber:integer);

begin

FHTTPRequests_DBGalleryViewer.IsHTTPConnectionOk();

  if not IsAllDownloadThreadsFinished then begin
  ShowMessage('  ,     ');
  exit;
  end;



  if Assigned(FDBPagination) then FreeAndNil(FDBPagination);
  FDBPagination:=TDBPagination.Create(Self);

  SetupDBPaginationModule();


  DBPagination.LoadPage(APageNumber); // << opens query, load records

  UpdatePagesLabel();
  UpdateObjectListOfPictures();
  FGalleryOfPictureFrames.DisposePicturesOnGalleryFrame();




end;



procedure TMainVisualFrame.LoadNextPage;

begin

if not IsAllDownloadThreadsFinished then begin
ShowMessage('  ,     ');
exit;
end;


if DBPagination.CurrentPage<DBPagination.CountOfPages then

  begin


  DBPagination.LoadNextPage(); // << opens query, load records

  UpdatePagesLabel();
  UpdateObjectListOfPictures();
  FGalleryOfPictureFrames.DisposePicturesOnGalleryFrame();



  end;

end;


procedure TMainVisualFrame.LoadPreviousPage;

begin

if not IsAllDownloadThreadsFinished then begin
ShowMessage('  ,     ');
exit;
end;



  if DBPagination.CurrentPage>1 then begin


      DBPagination.LoadPreviousPage(); // << opens query, load records
      UpdatePagesLabel();
      UpdateObjectListOfPictures();
      FGalleryOfPictureFrames.DisposePicturesOnGalleryFrame();



  end;


end;





procedure TMainVisualFrame.sePagesChange(Sender: TObject);
begin
{ this work doesn't work correctly
if sePages.Value>DBPagination.CountOfPages then
sePages.Value:=DBPagination.CountOfPages;

if sePages.Value<1 then sePages.Value:=1;
}

end;

procedure TMainVisualFrame.UpdateNumberOfRecords();
begin

FDBPagination.RecordsOnPage :=cbElementsOnPage.Items[cbElementsOnPage.ItemIndex].ToInteger;
FDBPagination.CountNumberOfPages;

end;

procedure TMainVisualFrame.SortIfNeeded;
var
  sortParam: string;
  sortDirection: string;
begin
  // sorting if needed...
  if FSorted then
  begin
    sortParam := '';
    sortDirection := '';
    //sortParam
    case cbSortParam.ItemIndex of
      0:
        sortParam := 'name';
      1:
        sortParam := 'fileSize';
      2:
        sortParam := 'creationDateTime';
      3:
        sortParam := 'changesDateTime';
      4:
        sortParam := 'lastAccessDateTime';
    end;
    //sortDirection
    case cbSortDirection.ItemIndex of
      0:
        sortDirection := 'ASC';
      1:
        sortDirection := 'DESC';
    end;
    FDBPagination.InitialSQLText := FDBPagination.InitialSQLText + ' Order By ' + sortParam + ' ' + sortDirection;
  end;
end;





procedure TMainVisualFrame.SetupDBPaginationModule();


begin


  if Assigned(FDBPagination) then

  begin


       with FDBPagination do begin

        // connecting External queries qMain and qMainCount of Records
        qMain.Connection:=FQueries.qSelectAllFilesOfUser.Connection;
        qMain.SQL.Text:=FQueries.qSelectAllFilesOfUser.SQL.Text;
        InitialSQLText:=FQueries.qSelectAllFilesOfUser.SQL.Text; // will be changed in process()

        qMainCountOfRecords:=FQueries.qSelectCountAllFilesOfUser;


         // transferring IDUSer value
        IsSomeUser:=true;
        IDUser:=FHTTP_ConnectionParams.IDUser.ToInteger;
        AddUsersParamToMainQuery();

       end;

       // sorting
       SortIfNeeded();

      // counting number of pages
      UpdateNumberOfRecords();
      
    end;


end;



procedure TMainVisualFrame.UpdateObjectListOfPictures();
var
  LastAddedIndex: Integer;
  relPath: String;
  id:integer;
  ObjectName: String;

  LastAddedThreadIndex:integer;

begin
  {
for i := 0  to DBPagination.RecordsOnPage-1 do begin
//  FGalleryOfPicturesFrames

end;
}         ProgressBar.Position:=0;

//          DBPagination.CountNumberOfPages;
//          DBPagination.CountNumberOfRecordsOnLastPage;

          if DBPagination.CurrentPage<>DBPagination.CountOfPages then
          ProgressBar.Max:=DBPagination.RecordsOnPage else
          ProgressBar.Max:=DBPagination.RecordsOnLastPage;

               // clearing and filling new list
          FGalleryOfPictureFrames.PictureFramesOL.Clear;

          with FDBPagination.qMain do //FQueries.qSelectAllFilesOfUser do
          begin

            while not EOF do
            begin
              LastAddedIndex := FGalleryOfPictureFrames.AddPictureFrame;

              with TPictureFrame(FGalleryOfPictureFrames.PictureFramesOL[LastAddedIndex]) do
              begin

                 id:=FieldByName('id').AsInteger;//FQueries.qSelectAllFilesOfUser.FieldByName('id').AsInteger;

              //do something here with frames...
                   if not FCheckFileType.IsFolder(id) then begin

                    relPath:=FieldByName('relPath').AsString;;//FQueries.qSelectAllFilesOfUser.FieldByName('relPath').AsString;

                    LastAddedThreadIndex:=FDownloadThreadsOL.Add(TDownloadPictureThread.Create(true));

                     with TDownloadPictureThread(FDownloadThreadsOL[LastAddedThreadIndex]) do begin

                     MainVisualFrame:=Self;
                     //ObjectName:=FQueries.qSelectAllFilesOfUser.FieldByName('name').AsString;
                     FreeOnTerminate:=false;
                     PictureFrame:=TPictureFrame(FGalleryOfPictureFrames.PictureFramesOL[LastAddedIndex]);
                     RelativeLink:=relPath;
                     Start;

                     end;

                     ObjectName:=FieldByName('name').AsString;//FQueries.qSelectAllFilesOfUser.FieldByName('name').AsString;

                     Hint:=ObjectName; ShowHint:=true;

                     if ObjectName.Length>15 then ObjectName:=ObjectName.Substring(0,15)+'...';
                     cbCheckBox.Caption:=ObjectName;



                   end else

                    begin

                    // do something with folders

                    end;


                 // if is picture by dataBase

              end;

              Next();
            end;
          end;





end;

procedure TMainVisualFrame.DisposeGalleryOfPictureFrames;
begin

  FGalleryOfPictureFrames.Align:=alTop;

//  FGalleryOfPictureFrames.Left := ScrollBox.Left;
//  FGalleryOfPictureFrames.Top := ScrollBox.Top;

  FGalleryOfPictureFrames.Width := ScrollBox.Width;
  FGalleryOfPictureFrames.Height := ScrollBox.Height;
end;



procedure TMainVisualFrame.eSearchClick(Sender: TObject);
begin

 if not (FFirstSearchPhraseDeleted) and 

 (eSearch.Text='    ...') 

 then 

   begin
   FFirstSearchPhraseDeleted:=true;
   eSearch.Text:='';
   end;
 
 

end;

procedure TMainVisualFrame.UpdatePagesLabel;
begin
  lPages.Caption := FDBPagination.CurrentPage.ToString + ' / ' + FDBPagination.CountOfPages.ToString;
  sePages.Value:=FDBPagination.CurrentPage;
end;

end.
