unit uDBPagination;

{< DB pagination for mySQL - adding LIMIT M,N phrase to Select Request}

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error,
  FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, Data.DB, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client,Math;

type

  TDBPagination = class(TDataModule)
    qMain: TFDQuery;
    qMainCountOfRecords: TFDQuery;
  private

  // FFDQuery:TFDQuery;

   FCountOfRecords:Int64;

   FCurrentRow:Int64;
   FCurrentPage:Integer;
   FCountOfPages:Integer;


   FRecordsOnPage:Int64;
   FInitialSQLText:string;



   FRecordsOnLastPage:int64;

   FIDUser:integer;
   FIsSomeUser:Boolean;

  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;

    procedure LoadSomeLimitedNumberOfRows(AStartRow:Integer; ACountOfRows:integer);
    procedure CountNumberOfPages();
    procedure CountNumberOfRecordsOnLastPage;

    procedure LoadFirstPage();

    procedure LoadPage(APageNumber:integer);

    procedure LoadNextPage();
    procedure LoadPreviousPage();

    procedure AddUsersParamToMainQuery();

//    property FDQuery:TFDQuery read FFDQuery write FFDQuery;



    property CurrentPage:Integer read FCurrentPage;
    property CountOfPages:Integer read FCountOfPages;



    property  RecordsOnPage:Int64 read FRecordsOnPage write FRecordsOnPage;
    property  RecordsOnLastPage:int64 read FRecordsOnLastPage write FRecordsOnLastPage;


    property InitialSQLText:string read FInitialSQLText write FInitialSQLText;

    property CountOfRecords:Int64 read FCountOfRecords write FCountOfRecords;

    property IDUser:integer read FIDUser write FIDUser;

    property IsSomeUser:Boolean read FIsSomeUser write FIsSomeUser;

  end;


//var
//  DBPagination: TDBPagination;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

{ TDBPagination }

procedure TDBPagination.AddUsersParamToMainQuery();

var users_id_Param:TFDParam;

begin

 if IsSomeUser then begin


   users_id_Param:=qMain.Params.Add();

    with users_id_Param do begin

    Name:='users_id';
    ParamType:=ptInput;
    DataType:=ftInteger;


    end;



 end;

end;




procedure TDBPagination.CountNumberOfPages;
begin

Assert(FInitialSQLText<>'','FInitialSQLText is empty. Fill it First');
Assert(FRecordsOnPage<>0,'FRecordsOnPage is 0. Cannot be so Fill it First');

  if Assigned(qMain) then begin

      with qMainCountOfRecords do begin

      Disconnect();
      //SQL.Text:=FInitialSQLText;

      if IsSomeUser then Params.ParamValues['users_id']:=FIDUser;


      Open();

      FCountOfRecords:=FieldByName('CountOfRecords').AsLargeInt;

      FCountOfPages:=Ceil(FCountOfRecords/FRecordsOnPage);

      Disconnect();
      end;

   FRecordsOnLastPage:=FCountOfRecords-(FCountOfPages-1)*FRecordsOnPage;

  end;




end;


procedure TDBPagination.CountNumberOfRecordsOnLastPage;
begin

FRecordsOnLastPage:=FCountOfRecords-FCountOfPages*FRecordsOnPage;

end;

//


constructor TDBPagination.Create(AOwner: TComponent);
begin
  inherited;

  FCountOfPages:=0;
  FCurrentPage:=1;
  FInitialSQLText:='';

end;

procedure TDBPagination.LoadFirstPage;
begin

FCurrentPage:=1;
LoadSomeLimitedNumberOfRows(0,FRecordsOnPage);

end;

procedure TDBPagination.LoadSomeLimitedNumberOfRows(AStartRow:Integer; ACountOfRows:integer);


begin

    if Assigned(qMain) then

     begin

        qMain.Disconnect();

        // Attention here ! Original statement FDQuery.SQL.Text
        // should be without semicolon, also without params
        qMain.SQL.Text:=FInitialSQLText;
        qMain.SQL.Text:=qMain.SQL.Text+' LIMIT :StartRow,:CountOfRows';


        with qMain do begin

        if FIsSomeUser then Params.ParamValues['users_id']:=FIDUser;

        Params.ParamValues['StartRow']:=AStartRow;
        Params.ParamValues['CountOfRows']:=ACountOfRows;


        end;

        //FDQuery.IndexFieldNames:='fileSize:D';

       qMain.Open();


     end;


 end;





procedure TDBPagination.LoadNextPage();
begin

  if FCurrentPage<FCountOfPages then
  begin
  FCurrentPage:=FCurrentPage+1;
  FCurrentRow:=FCurrentRow+FRecordsOnPage;
  LoadSomeLimitedNumberOfRows(FCurrentRow,FRecordsOnPage);

  end;

end;

procedure TDBPagination.LoadPage(APageNumber:integer);
begin
//

if APageNumber>=1 then
begin
LoadSomeLimitedNumberOfRows((APageNumber-1)*FRecordsOnPage,FRecordsOnPage);
FCurrentPage:=APageNumber;
end;


end;

procedure TDBPagination.LoadPreviousPage();
begin

  if FCurrentPage>1 then
  begin

  FCurrentPage:=FCurrentPage-1;
  FCurrentRow:=FCurrentRow-FRecordsOnPage;
  LoadSomeLimitedNumberOfRows(FCurrentRow,FRecordsOnPage);

  end;

end;


end.
