unit uDownLoadPictureThread_DBGalleryViewer;

interface

uses

  Winapi.Windows,  Winapi.Messages,  System.SysUtils,  System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms,  Vcl.Dialogs,  Vcl.StdCtrls,
  Vcl.ComCtrls,  Vcl.ExtCtrls,  System.ImageList,  Vcl.ImgList,System.NetEncoding,
  System.SyncObjs,

  // myUnits
  uMainVisualFrame,
  uPictureFrame
  ;

type
  TDownloadPictureThread = class(TThread)
  private
    { Private declarations }
    FMainVisualFrame:TMainVisualFrame;
    FBitmapDownLoaded:TBitMap;
    FPictureFrame:TPictureFrame;

    FRelativeLink:String;

    FObjectName:String;

    fEx : Exception;
    procedure QueryError;

    procedure DownloadPicture(RelativeLinkToPicture:String);



  protected
    procedure Execute; override;

  public

  property MainVisualFrame:TMainVisualFrame read FMainVisualFrame write FMainVisualFrame;

  property RelativeLink:String read FRelativeLink write FRelativeLink;

  property PictureFrame:TPictureFrame read FPictureFrame write FPictureFrame;

  property ObjectName:String read FObjectName write FObjectName;

  end;

implementation

{
  Important: Methods and properties of objects in visual components can only be
  used in a method called using Synchronize, for example,

      Synchronize(UpdateCaption);  

  and UpdateCaption could look like,

    procedure DownloadPictureThread.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end; 
    
    or 
    
    Synchronize( 
      procedure 
      begin
        Form1.Caption := 'Updated in thread via an anonymous method' 
      end
      )
    );
    
  where an anonymous method is passed.
  
  Similarly, the developer can call the Queue method with similar parameters as 
  above, instead passing another TThread class as the first parameter, putting
  the calling thread in a queue with the other thread.
    
}

{ DownloadPictureThread }

procedure TDownloadPictureThread.DownloadPicture(RelativeLinkToPicture: String);

var Stream: TMemoryStream;
    twic:twicImage;
    Picture:TPicture;

    EncodedRelativeLink:String;


begin

//

  Stream:=TMemoryStream.Create;
  twic:=twicImage.Create;
  Picture:=TPicture.Create;




  try

  //
   EncodedRelativeLink:=RelativeLinkToPicture.Substring(1); // removing first slash
   EncodedRelativeLink:=TNetEncoding.URL.Encode(EncodedRelativeLink);


        EncodedRelativeLink:=

     FMainVisualFrame.HTTP_ConnectionParams.HTTPServerToRequest +'/'+ EncodedRelativeLink;

      with FMainVisualFrame do begin

       IdHTTPDownloadPicture.Get(EncodedRelativeLink,Stream);
       Stream.Position:=0;



       twic.LoadFromStream(Stream);
       FBitmapDownLoaded.Assign(twic);



      end;




  finally

  FreeAndNil(Stream);
  FreeAndNil(Twic);
  FreeAndNil(Picture);


  end;


end;

procedure TDownloadPictureThread.Execute;


begin
  { Place thread code here }

FMainVisualFrame.CSDownloadPictureThread.Enter;

FBitmapDownLoaded:=TBitmap.Create;

try

        try



          if FMainVisualFrame.CheckFileType.IsFilePicture(FRelativeLink) then begin




                                Synchronize(
                              procedure
                              begin
                              DownloadPicture(FRelativeLink);

                                PictureFrame.Image.Picture.Bitmap:=nil;
                                PictureFrame.Image.Picture.Bitmap.Assign(FBitmapDownLoaded);


                                if (PictureFrame.Image.Picture.Bitmap.Width<128)
                                or (PictureFrame.Image.Picture.Bitmap.Width<128)

                                then PictureFrame.Image.Stretch:=False else
                                PictureFrame.Image.Stretch:=true;


                              end
                              );




          end else if FMainVisualFrame.CheckFileType.IsFileVideo(FRelativeLink) then begin

               Synchronize(
                              procedure
                              begin

             PictureFrame.Image.Picture.Bitmap:=nil;
             MainVisualFrame.ilDefaultPics.GetBitmap(1,PictureFrame.Image.Picture.Bitmap);

                              end)



          end else if FMainVisualFrame.CheckFileType.IsFileAudio(FRelativeLink) then begin

                       Synchronize(
                              procedure
                              begin

            PictureFrame.Image.Picture.Bitmap:=nil;
             MainVisualFrame.ilDefaultPics.GetBitmap(2,PictureFrame.Image.Picture.Bitmap)

                              end)



          end else begin


                         Synchronize(
                              procedure
                              begin

             PictureFrame.Image.Picture.Bitmap:=nil;
             MainVisualFrame.ilDefaultPics.GetBitmap(0,PictureFrame.Image.Picture.Bitmap);

                              end)


          end;

                             Synchronize(
                              procedure

                              begin

                              MainVisualFrame.ProgressBar.Position:=(MainVisualFrame.ProgressBar.Position+1);

                              end)





        except

         FMainVisualFrame.CSDownloadPictureThread.Leave;

               Synchronize(
                              procedure
                              begin

                              PictureFrame.Image.Picture.Bitmap:=nil;
                              MainVisualFrame.ilDefaultPics.GetBitmap(0,PictureFrame.Image.Picture.Bitmap);

                              end);

        fEx := ExceptObject as Exception;
                Synchronize( QueryError );

        end;


finally

FreeAndNIl(FBitmapDownLoaded);
FMainVisualFrame.CSDownloadPictureThread.Leave;

end;

end;

procedure TDownloadPictureThread.QueryError;
begin
Application.OnException( Self, fEx );
end;

end.
